##################### generated by xml-casa (v2) from gaincal.xml ###################
##################### 4b8a0e9889f4fac3a7bbdf7e05c0ba89 ##############################
from __future__ import absolute_import
import numpy
from casatools.typecheck import CasaValidator as _val_ctor
_pc = _val_ctor( )
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from .private.task_gaincal import gaincal as _gaincal_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from casatasks.private.task_logging import except_log as _except_log

class _gaincal:
    """
    gaincal ---- Determine temporal gains from calibrator observations

    
    The complex gains for each antenna/spwid are determined from the data
    column (raw data) divided by the model column. The gains can be
    obtained for a specified solution interval, spw combination and field
    combination. The GSPLINE spline (smooth) option is still under
    development.
    
    Previous calibrations (egs, bandpass, opacity, parallactic angle) can
    be applied on the fly. At present with dual-polarized data, both
    polarizations must be unflagged for any solution to be obtained.

    --------- parameter descriptions ---------------------------------------------

    vis          Name of input visibility file
                 Default: none
                 
                    Example: vis='ngc5921.ms'
    caltable     Name of output gain calibration table
                 Default: none
                 
                    Example: caltable='ngc5921.gcal'
    field        Select field using field id(s) or field name(s)
                 Default: '' (all fields)
                 
                 Use 'go listobs' to obtain the list id's or
                 names. If field string is a non-negative integer,
                 it is assumed a field index,  otherwise, it is
                 assumed a field name.
                 
                    Examples:
                    field='0~2'; field ids 0,1,2
                    field='0,4,5~7'; field ids 0,4,5,6,7
                    field='3C286,3C295'; field named 3C286 and
                    3C295
                    field = '3,4C\*'; field id 3, all names
                    starting with 4C
    spw          Select spectral window/channels
                 Default: '' (all spectral windows and channels)
                 
                    Examples:
                    spw='0~2,4'; spectral windows 0,1,2,4 (all
                    channels)
                    spw='<2';  spectral windows less than 2
                    (i.e. 0,1)
                    spw='0:5~61'; spw 0, channels 5 to 61,
                    INCLUSIVE
                    spw='\*:5~61'; all spw with channels 5 to 61
                    spw='0,10,3:3~45'; spw 0,10 all channels, spw
                    3, channels 3 to 45.
                    spw='0~2:2~6'; spw 0,1,2 with channels 2
                    through 6 in each.
                    spw='0:0~10;15~60'; spectral window 0 with
                    channels 0-10,15-60. (NOTE ';' to separate
                    channel selections)
                    spw='0:0~10^2,1:20~30^5'; spw 0, channels
                    0,2,4,6,8,10, spw 1, channels 20,25,30
    intent       Select observing intent
                 Default: '' (no selection by intent)
                 
                    Example: intent='*BANDPASS*'  (selects data
                    labelled with BANDPASS intent)
    selectdata   Other data selection parameters
                 Default: True
                 Options: True|False
    timerange    Select data based on time range
                 Subparameter of selectdata=True
                 Default = '' (all)
                 
                    Examples:
                    timerange =
                    'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
                    (Note: if YYYY/MM/DD is missing date defaults
                    to first day in data set.)
                    timerange='09:14:0~09:54:0' picks 40 min on
                    first day 
                    timerange= '25:00:00~27:30:00' picks 1 hr to 3
                    hr 30min on NEXT day
                    timerange='09:44:00' pick data within one
                    integration of time
                    timerange='>10:24:00' data after this time
    uvrange      Select data by baseline length.
                 Default = '' (all)
                 
                    Examples:
                    uvrange='0~1000klambda'; uvrange from 0-1000 kilo-lambda
                    uvrange='>4klambda';uvranges greater than 4 kilo-lambda
                    uvrange='0~1000km'; uvrange in kilometers
    antenna      Select data based on antenna/baseline
                                    Subparameter of selectdata=True
                                    Default: '' (all)
                 
                                    If antenna string is a non-negative integer, it
                                    is assumed an antenna index, otherwise, it is
                                    assumed as an antenna name
                 
                                        Examples: 
                                        antenna='5&6'; baseline between antenna
                                        index 5 and index 6.
                                        antenna='VA05&VA06'; baseline between VLA
                                        antenna 5 and 6.
                                        antenna='5&6;7&8'; baselines with
                                        indices 5-6 and 7-8
                                        antenna='5'; all baselines with antenna index
                                        5
                                        antenna='05'; all baselines with antenna
                                        number 05 (VLA old name)
                                        antenna='5,6,10'; all baselines with antennas
                                        5,6,10 index numbers
    scan         Scan number range
                 Subparameter of selectdata=True
                 Default: '' = all
                 
                 Check 'go listobs' to insure the scan numbers are
                 in order.
    observation  Select by observation ID(s)
                 Subparameter of selectdata=True
                 Default: '' = all
                 
                     Example: observation='0~2,4'
    msselect     Optional complex data selection (ignore for now)
    solint       Solution interval
                 Default: 'inf' (infinite, up to boundaries
                 controlled by combine); 
                 Options: 'inf' (~infinite), 'int' (per
                 integration), any float or integer value with or
                 without units
                 
                    Examples: 
                    solint='1min'; solint='60s', solint=60 (i.e.,
                    1 minute); solint='0s'; solint=0; solint='int'
                    (i.e., per integration); solint-'-1s';
                    solint='inf' (i.e., ~infinite, up to
                    boundaries enforced by combine)
    combine      Data axes which to combine for solve
                 Default: 'scan' (solutions will break at obs,
                 field, and spw boundaries)
                 Options: '','obs','scan','spw',field', or any
                 comma-separated combination in a single string
                 
                    Example: combine='scan,spw' - Extend solutions
                    over scan boundaries (up to the solint), and
                    combine spws for solving
    preavg       Pre-averaging interval (sec)
                 Default: -1.0 (none)
                 
                 Rarely needed.  Will average data over periods
                 shorter than the solution interval first.
    refant       Reference antenna name(s); a prioritized list may be
                 specified
                 Default: '' (No refant applied)
                 
                    Examples: 
                    refant='4' (antenna with index 4)
                    refant='VA04' (VLA antenna #4)
                    refant='EA02,EA23,EA13' (EVLA antenna EA02,
                    use EA23 and EA13 as alternates if/when EA02
                    drops out)
                 
                 Use taskname=listobs for antenna listing
    refantmode   Reference antenna mode
    minblperant  Minimum number of baselines required per antenna for each
                 solve
                 Default: 4
                 
                 Antennas with fewer baselines are excluded from
                 solutions.
                 
                    Example: minblperant=10 --> Antennas
                    participating on 10 or more baselines are
                    included in the solve
                 
                 minblperant = 1 will solve for all baseline
                 pairs, even if only one is present in the data
                 set.  Unless closure errors are expected, use
                 taskname=gaincal rather than taskname=blcal to
                 obtain more options in data analysis.
    minsnr       Reject solutions below this SNR
                 Default: 3.0
    solnorm      Normalize (squared) solution amplitudes (G, T only)
                 Default: False (no normalization)
    normtype     Solution normalization calculation type: mean or median
                 Default: 'mean'
    gaintype     Type of gain solution (G,T,GSPLINE,K,KCROSS)
                   Default: 'G'
                 
                      Example: gaintype='GSPLINE'
                 
                 - 'G' means determine gains for each polarization and sp_wid
                 - 'T' obtains one solution for both polarizations;
                   Hence. their phase offset must be first removed
                   using a prior G.
                 - 'GSPLINE' makes a spline fit to the calibrator
                   data. It is useful for noisy data and fits a
                   smooth curve through the calibrated amplitude and
                   phase. However, at present GSPLINE is somewhat
                   experimental. Use with caution and check
                   solutions.
                 - 'K' solves for simple antenna-based delays via
                   FFTs of the spectra on baselines to the reference
                   antenna.  (This is not global fringe-fitting.)
                   If combine includes 'spw', multi-band delays are
                   determined; otherwise, per-spw single-band delays
                   will be determined.
                 - 'KCROSS' solves for a global cross-hand delay.
                   Use parang=T and apply prior gain and bandpass
                   solutions.  Multi-band delay solves
                   (combine='spw') not yet supported for KCROSS.
    smodel       Point source Stokes parameters for source model
                 (experimental).
                 Default: [] (use MODEL_DATA column)
                 
                    Example: [1,0,0,0] (I=1, unpolarized)
    calmode      Type of solution" ('ap', 'p', 'a')
                 Default: 'ap' (amp and phase)
                 Options: 'p' (phase) ,'a' (amplitude), 'ap'
                 (amplitude and phase)
                 
                    Example: calmode='p'
    solmode      Robust solving mode: 
                 Options: '', 'L1', 'R', 'L1R'
    rmsthresh    RMS Threshold sequence
                 Subparameter of solmode='R' or 'L1R'
                 
                 See CASA Docs for more information
                 (https://casa.nrao.edu/casadocs/)
    corrdepflags f False (default), if any correlation is flagged, treat all correlations in
                             the visibility vector as flagged when solving (per channel, per baseline).
                             If True, use unflagged correlations in a visibility vector, even if one or more
                             other correlations are flagged.
                             
                                    Default: False (treat correlation vectors with one or more correlations flagged as entirely flagged)
                 
                                    Traditionally, CASA has observed a strict interpretation of 
                                    correlation-dependent flags: if one or more correlations 
                                    (for any baseline and channel) is flagged, then all available 
                                    correlations for the same baseline and channel are 
                                    treated as flagged.  However, it is desirable in some 
                                    circumstances to relax this stricture, e.g., to preserve use
                                    of data from antennas with only one good polarization (e.g., one polarization
                                    is bad or entirely absent).  Solutions for the bad or missing polarization 
                                    will be rendered as flagged.
    append       Append solutions to the (existing) table
                 Default: False (overwrite existing table or make
                 new table)
                 
                 Appended solutions must be derived from the same
                 MS as the existing caltable, and solution spws
                 must have the same meta-info (according to spw
                 selection and solint) or be non-overlapping.
    splinetime   Spline timescale(sec); All spw\'s are first averaged.
                 Subparameter of gaintype='GSPLINE'
                 Default: 3600 (1 hour)
                 
                    Example: splinetime=1000
                 
                 Typical splinetime should cover about 3 to 5
                 calibrator scans.
    npointaver   Tune phase-unwrapping algorithm
                 Subparameter of gaintype='GSPLINE'
                 Default: 3; Keep at this value
    phasewrap    Wrap the phase for jumps greater than this value
                 (degrees)
                 Subparameter of gaintype='GSPLINE'
                 Default: 180; Keep at this value
    docallib     Control means of specifying the caltables
                 Default: False (Use gaintable, gainfield, interp,
                 spwmap, calwt)
                 Options: False|True
                 
                 If True, specify a file containing cal library in
                 callib
    callib       Specify a file containing cal library directives
                 Subparameter of docallib=True
    gaintable    Gain calibration table(s) to apply on the fly
                 Default: '' (none)
                 Subparameter of docallib=False
                    Examples: 
                    gaintable='ngc5921.gcal'
                    gaintable=['ngc5921.ampcal','ngc5921.phcal']
    gainfield    Select a subset of calibrators from gaintable(s)
                 Default: '' (all sources on the sky)
                 
                 'nearest' ==> nearest (on sky) available field in
                 table otherwise, same syntax as field
                 
                    Examples: 
                    gainfield='0~2,5' means use fields 0,1,2,5
                    from gaintable
                    gainfield=['0~3','4~6'] means use field 0
                    through 3
    interp       Interpolation parmameters (in time[,freq]) for each gaintable, as a list of strings.
                   Default: '' --> 'linear,linear' for all gaintable(s)
                   Options: Time: 'nearest', 'linear'
                            Freq: 'nearest', 'linear', 'cubic',
                            'spline'
                 Specify a list of strings, aligned with the list of caltable specified
                 in gaintable, that contain the required interpolation parameters
                 for each caltable.
                 
                 - When frequency interpolation is relevant (B, Df,
                   Xf), separate time-dependent and freq-dependent
                   interp types with a comma (freq after the
                   comma). 
                 - Specifications for frequency are ignored when the
                   calibration table has no channel-dependence. 
                 - Time-dependent interp options ending in 'PD'
                   enable a "phase delay" correction per spw for
                   non-channel-dependent calibration types.
                 - For multi-obsId datasets, 'perobs' can be
                   appended to the time-dependent interpolation
                   specification to enforce obsId boundaries when
                   interpolating in time. 
                 - For multi-scan datasets, 'perscan' can be
                   appended to the time-dependent interpolation
                   specification to enforce scan boundaries when
                   interpolating in time. 
                 - Freq-dependent interp options can have 'flag' appended
                   to enforce channel-dependent flagging, and/or 'rel' 
                   appended to invoke relative frequency interpolation
                 
                      Examples: 
                      interp='nearest' (in time, freq-dep will be
                      linear, if relevant)
                      interp='linear,cubic'  (linear in time, cubic
                      in freq)
                      interp='linearperobs,splineflag' (linear in
                      time per obsId, spline in freq with
                      channelized flagging)
                      interp='nearest,linearflagrel' (nearest in
                      time, linear in freq with with channelized 
                      flagging and relative-frequency interpolation)
                      interp=',spline'  (spline in freq; linear in
                      time by default)
                      interp=['nearest,spline','linear']  (for
                      multiple gaintables)
    spwmap       Spectral window mappings to form for gaintable(s)
                 Only used if callib=False
                 default: [] (apply solutions from each calibration spw to
                  the same MS spw only)
                 Any available calibration spw can be mechanically mapped to any 
                  MS spw. 
                 Examples:
                    spwmap=[0,0,1,1] means apply calibration 
                      from cal spw = 0 to MS spw 0,1 and cal spw 1 to MS spws 2,3.
                    spwmap=[[0,0,1,1],[0,1,0,1]] (use a list of lists for multiple
                      gaintables)
    parang       Apply parallactic angle correction
                 Default: False
                 
                 If True, apply the parallactic angle correction
                 (required for polarization calibration)

    --------- examples -----------------------------------------------------------

    
    
    For more information, see the task pages of gaincal in CASA Docs:
    
    https://casa.nrao.edu/casadocs/


    """

    _info_group_ = """calibration"""
    _info_desc_ = """Determine temporal gains from calibrator observations"""

    def __call__( self, vis='', caltable='', field='', spw='', intent='', selectdata=True, timerange='', uvrange='', antenna='', scan='', observation='', msselect='', solint='inf', combine='', preavg=float(-1.0), refant='', refantmode='flex', minblperant=int(4), minsnr=float(3.0), solnorm=False, normtype='mean', gaintype='G', smodel=[  ], calmode='ap', solmode='', rmsthresh=[  ], corrdepflags=False, append=False, splinetime=float(3600.0), npointaver=int(3), phasewrap=float(180.0), docallib=False, callib='', gaintable=[  ], gainfield=[  ], interp=[  ], spwmap=[ ], parang=False ):
        schema = {'vis': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'caltable': {'type': 'cStr', 'coerce': _coerce.to_str}, 'field': {'type': 'cStr', 'coerce': _coerce.to_str}, 'spw': {'type': 'cStr', 'coerce': _coerce.to_str}, 'intent': {'type': 'cStr', 'coerce': _coerce.to_str}, 'selectdata': {'type': 'cBool'}, 'timerange': {'type': 'cStr', 'coerce': _coerce.to_str}, 'uvrange': {'type': 'cVariant', 'coerce': [_coerce.to_variant]}, 'antenna': {'type': 'cStr', 'coerce': _coerce.to_str}, 'scan': {'type': 'cStr', 'coerce': _coerce.to_str}, 'observation': {'anyof': [{'type': 'cStr', 'coerce': _coerce.to_str}, {'type': 'cInt'}]}, 'msselect': {'type': 'cStr', 'coerce': _coerce.to_str}, 'solint': {'type': 'cVariant', 'coerce': [_coerce.to_variant]}, 'combine': {'type': 'cStr', 'coerce': _coerce.to_str}, 'preavg': {'type': 'cFloat', 'coerce': _coerce.to_float}, 'refant': {'type': 'cStr', 'coerce': _coerce.to_str}, 'refantmode': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'flex', 'strict' ]}, 'minblperant': {'type': 'cInt'}, 'minsnr': {'type': 'cFloat', 'coerce': _coerce.to_float}, 'solnorm': {'type': 'cBool'}, 'normtype': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'mean', 'median' ]}, 'gaintype': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'XYf+QU', 'XY+QU', 'K', 'KCROSS', 'T', 'G', 'GSPLINE' ]}, 'smodel': {'type': 'cFloatVec', 'coerce': [_coerce.to_list,_coerce.to_floatvec]}, 'calmode': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'a', 'p', 'ap' ]}, 'solmode': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ '', 'L1', 'L1R', 'R' ]}, 'rmsthresh': {'type': 'cFloatVec', 'coerce': [_coerce.to_list,_coerce.to_floatvec]}, 'corrdepflags': {'type': 'cBool'}, 'append': {'type': 'cBool'}, 'splinetime': {'type': 'cFloat', 'coerce': _coerce.to_float}, 'npointaver': {'type': 'cInt'}, 'phasewrap': {'type': 'cFloat', 'coerce': _coerce.to_float}, 'docallib': {'type': 'cBool'}, 'callib': {'type': 'cStr', 'coerce': _coerce.to_str}, 'gaintable': {'type': 'cStrVec', 'coerce': [_coerce.to_list,_coerce.to_strvec]}, 'gainfield': {'type': 'cStrVec', 'coerce': [_coerce.to_list,_coerce.to_strvec]}, 'interp': {'type': 'cStrVec', 'coerce': [_coerce.to_list,_coerce.to_strvec]}, 'spwmap': {'type': 'cVariant', 'coerce': [_coerce.to_variant]}, 'parang': {'type': 'cBool'}}
        doc = {'vis': vis, 'caltable': caltable, 'field': field, 'spw': spw, 'intent': intent, 'selectdata': selectdata, 'timerange': timerange, 'uvrange': uvrange, 'antenna': antenna, 'scan': scan, 'observation': observation, 'msselect': msselect, 'solint': solint, 'combine': combine, 'preavg': preavg, 'refant': refant, 'refantmode': refantmode, 'minblperant': minblperant, 'minsnr': minsnr, 'solnorm': solnorm, 'normtype': normtype, 'gaintype': gaintype, 'smodel': smodel, 'calmode': calmode, 'solmode': solmode, 'rmsthresh': rmsthresh, 'corrdepflags': corrdepflags, 'append': append, 'splinetime': splinetime, 'npointaver': npointaver, 'phasewrap': phasewrap, 'docallib': docallib, 'callib': callib, 'gaintable': gaintable, 'gainfield': gainfield, 'interp': interp, 'spwmap': spwmap, 'parang': parang}
        assert _pc.validate(doc,schema), create_error_string(_pc.errors)
        _logging_state_ = _start_log( 'gaincal', [ 'vis=' + repr(_pc.document['vis']), 'caltable=' + repr(_pc.document['caltable']), 'field=' + repr(_pc.document['field']), 'spw=' + repr(_pc.document['spw']), 'intent=' + repr(_pc.document['intent']), 'selectdata=' + repr(_pc.document['selectdata']), 'timerange=' + repr(_pc.document['timerange']), 'uvrange=' + repr(_pc.document['uvrange']), 'antenna=' + repr(_pc.document['antenna']), 'scan=' + repr(_pc.document['scan']), 'observation=' + repr(_pc.document['observation']), 'msselect=' + repr(_pc.document['msselect']), 'solint=' + repr(_pc.document['solint']), 'combine=' + repr(_pc.document['combine']), 'preavg=' + repr(_pc.document['preavg']), 'refant=' + repr(_pc.document['refant']), 'refantmode=' + repr(_pc.document['refantmode']), 'minblperant=' + repr(_pc.document['minblperant']), 'minsnr=' + repr(_pc.document['minsnr']), 'solnorm=' + repr(_pc.document['solnorm']), 'normtype=' + repr(_pc.document['normtype']), 'gaintype=' + repr(_pc.document['gaintype']), 'smodel=' + repr(_pc.document['smodel']), 'calmode=' + repr(_pc.document['calmode']), 'solmode=' + repr(_pc.document['solmode']), 'rmsthresh=' + repr(_pc.document['rmsthresh']), 'corrdepflags=' + repr(_pc.document['corrdepflags']), 'append=' + repr(_pc.document['append']), 'splinetime=' + repr(_pc.document['splinetime']), 'npointaver=' + repr(_pc.document['npointaver']), 'phasewrap=' + repr(_pc.document['phasewrap']), 'docallib=' + repr(_pc.document['docallib']), 'callib=' + repr(_pc.document['callib']), 'gaintable=' + repr(_pc.document['gaintable']), 'gainfield=' + repr(_pc.document['gainfield']), 'interp=' + repr(_pc.document['interp']), 'spwmap=' + repr(_pc.document['spwmap']), 'parang=' + repr(_pc.document['parang']) ] )
        task_result = None
        try:
            task_result = _gaincal_t( _pc.document['vis'], _pc.document['caltable'], _pc.document['field'], _pc.document['spw'], _pc.document['intent'], _pc.document['selectdata'], _pc.document['timerange'], _pc.document['uvrange'], _pc.document['antenna'], _pc.document['scan'], _pc.document['observation'], _pc.document['msselect'], _pc.document['solint'], _pc.document['combine'], _pc.document['preavg'], _pc.document['refant'], _pc.document['refantmode'], _pc.document['minblperant'], _pc.document['minsnr'], _pc.document['solnorm'], _pc.document['normtype'], _pc.document['gaintype'], _pc.document['smodel'], _pc.document['calmode'], _pc.document['solmode'], _pc.document['rmsthresh'], _pc.document['corrdepflags'], _pc.document['append'], _pc.document['splinetime'], _pc.document['npointaver'], _pc.document['phasewrap'], _pc.document['docallib'], _pc.document['callib'], _pc.document['gaintable'], _pc.document['gainfield'], _pc.document['interp'], _pc.document['spwmap'], _pc.document['parang'] )
        except Exception as exc:
            _except_log('gaincal', exc)
            raise
        finally:
            task_result = _end_log( _logging_state_, 'gaincal', task_result )
        return task_result

gaincal = _gaincal( )

